package model;

import java.awt.Color;

import view.GSnip;

public enum Block {
	air(0, false, null, 0,0,0,0, 0),
	stone,
	player(0, true, null, 0.16D,0.16D,0.16D,0.16D, 0.15D),
	dirt,
	torch(0, false, null, 0.45d,0.45d,0.45d,0.45d, 0.30D); // TODO wire up torch

	private int data;								// at least 32bits of your own memory!
	public final boolean solid;					// can player walk through?
	public final GSnip icon;						// icon to appear in inventory
	public final double insl, insr, inst, insb;	// insets of the box (ie. set all to about 0.45d to get a centred stick(gonna use for torch) ) TODO make these affect anything actually...
	public final double light;						// how much light is being emmited. (0: none. 1:double sunlight?)

	Block(int data, boolean solid, GSnip icon, double insl, double insr, double inst, double insb, double light) {
		this.solid = solid;
		this.icon = icon;
		this.data = data;
		this.insl=insl;
		this.insr=insr;
		this.inst=inst;
		this.insb=insb;
		this.light = light;
	}
	Block() {
		solid = true;
		icon = Lib.icons.byBlock(this);
		data = 0;
		insl=0;
		insr=0;
		inst=0;
		insb=0;
		light=0D;
	}
	Block(int data) { this(); this.data = data; }
	
	public GSnip getIcon() {
		return icon;
	}
	
	public boolean isSolid() {
		return solid;
	}
	
	public int getData() { return data; }
	public void setData(int data) { this.data = data; }
	
	public String toString() {
		String ret;
		
		switch(this) {
		case stone:			ret = "stone"; break;
		case player:		ret = "player"; break;
		case dirt:			ret = "dirt"; break;
		default:			ret = "air";
		}
		
		if (data == 0) return ret;
		else           return ret+":"+data;
	}
	
	public String encode() { // TODO "4E 55 4D 42   44 41 54 41" (without spaces)
		String type = Integer.toHexString(this.ordinal());
		String data = Integer.toHexString(getData());
		
		int tlen = type.length();
		int dlen = data.length();
		
		for (int i=0; i<(8-tlen); i++) type = "0"+type;
		for (int i=0; i<(8-dlen); i++) data = "0"+data;
		
		return type+data;
	}
	public static Block decode(String wut) {
		Block b = Block.values()[Integer.parseInt(wut.substring(0, 8), 16)];
		b.setData(Integer.parseInt(wut.substring(9, 16), 16));
		return b;
	}
	
}
