package model;

import java.awt.Color;

import view.GraphicsEngine;
import mutil.*;

/**
 * 
 * pickup() return codes: 0/positive: okay. (index of slot) -1[egative]: error.
 */

public class EntityPlayer extends Entity {
	private double dir, speed;
	private int hp, maxhp;
	private int exp, maxexp;
	private int level;
	private Inventory inv;
	private Inventory palette;
	private Inventory armor;

	public EntityPlayer(World in, double x, double y) {
		super(in, x, y);
		levelup(1);
		inv = new Inventory(8 * 4);
		palette = new Inventory(8 * 1);
		armor = new Inventory(4 * 1);
	}

	public boolean isOpaque() { return true; }
	public double insL() { return 0.2; }
	public double insR() { return 0.2; }
	public double insT() { return 0.2; }
	public double insB() { return 0.2; }
	
	@Override
	public int act_Primary(double x, double y) {
		// TODO Auto-generated method stub
		return super.act_Primary(x, y);
	}
	
	public Inventory getInv() {
		return inv;
	}

	public Inventory getPalette() {
		return palette;
	}

	public Inventory getArmor() {
		return armor;
	}

	public int pickup(Object wut) {
		int ret;

		if ((ret = palette.push(wut)) < 0 && (ret = inv.push(wut)) < 0) {
			return -1;
		}

		return ret;
	}

	public void die() {
		System.out.printf("WORLD [Entity %d] (EntityPlayer) died!\n", eid);
		destroyme();
	}

	public void levelup(int levelnew) {
		level = levelnew;

		maxexp = 70 + 20 * (level - 1);
		exp = 1;
		maxhp = 100 + 10 * (level - 1);
		if (level == 1) hp = maxhp; else hp += maxhp / 10;
		
		if (hp > maxhp) {
			hp = maxhp;
		}
	}

	/**
	 * Hurt this player!
	 * @param ouchies how much damage points.
	 * @return hp left
	 */
	public int hurt(int ouchies) {
		this.hp -= ouchies;
		if (hp < 0) {
			hp = 0;
			die();
		}
		return hp;
	}

	public int getHP() {
		return hp;
	}

	public int getMaxHP() {
		return maxhp;
	}

	public double getHPRatio() {
		return (double) maxhp / (double) hp;
	}

	public int getExp() {
		return exp;
	}

	public int getMaxExp() {
		return maxexp;
	}

	public double getExpRatio() {
		return (double) exp / (double) maxexp;
	}

	public int getLevel() {
		return level;
	}

	@Override
	public void move(double x, double y) {
		Block b = world.get((int) x, (int) y);

		if (b.isSolid()) {
			return;
		}
		super.move(x, y);
	}

	@Override
	public @Undone
	void update(double delta) {
		x += Math.cos(dir) * speed * delta;
		y += Math.sin(dir) * speed * delta;

		// System.out.printf("WORLD [Entity Player: update] tick with delta=%f!\n",
		// delta);
	}

}
