package model;

import java.awt.Color;
import java.awt.image.BufferedImage;
import java.io.*;
import java.util.HashMap;
import java.util.List;
import java.util.LinkedList;
import java.util.Vector;

import javax.imageio.ImageIO;

import view.Compiz;
import view.GraphicsEngine;
import view.Snippet;
import view.GSnip;
import view.View;
import view.gui.Window;
import view.gui.WindowInventory;
import view.gui.WindowManager;

import java.io.File;
import java.io.IOException;
import java.util.HashSet;
import java.util.Random;
import java.util.Set;
import java.util.Stack;

public class Model {

	public Model() {
		propertifies = new HashMap<String, Object>(100);
		compizsnippets = new LinkedList<Snippet>();
		wm = new WindowManager();
		world = new World();

//		world = load("My save 1");
		player = new EntityPlayer(world, 0, 0);
		System.out
				.printf("CORE INIT [model.Model this(..) news] (EntityPlayer)player.eid = %d\n",
						player.getEID());

		BufferedImage img = null;
		try {
			img = ImageIO.read(new File(new File(".").getCanonicalPath() +
			"/worldfix_map_110529.bmp"));
			for (int x=0; x<img.getWidth(); x++) {
				for (int y=0; y<img.getHeight(); y++) {
					if (img.getRGB(x, y) == Color.white.getRGB()) world.set(x, y, Block.stone);
					if (img.getRGB(x, y) == Color.blue.getRGB()) new EntitySpawner(world, x,y, EntityMonsterMicrosoft.class);
					else world.set(x, y, Block.air);
				}
			}
		} catch (Exception ex) {
			System.out.println("Model.this(): couldnt load map"); }

		world.set(10, 20, Block.torch);
		new EntityMonster(world, 22, 22);

		ppalette = new WindowInventory(player.getPalette(), 8);
		ppalette.initWM(wm);
		ppalette.setCaption("Your palette");
		ppalette.resize(212, 27 + Window.frametop);

		termtext = new Vector<String>();
	}

	private WindowInventory ppalette;

	public final WindowInventory getPPalette() {
		return ppalette;
	}

	/** e.g. "bobs save" ((--> "./bobs save.sav") */
	public final void save(final String savename) {
		try {
			FileWriter fw = new FileWriter(savename + ".sav");
			BufferedWriter out = new BufferedWriter(fw);

			List<Chunk> chunks = world.getChunks();

			for (Chunk c : chunks) {

				out.write("!" + c.x + "," + c.y + "\n");

				for (int x = 0; x < Chunk.cs; x++) {
					for (int y = 0; y < Chunk.cs; y++) {
						Block b = c.getr(x, y);

						if (b != null) {
							out.write(b.encode());
						} else {
							out.write("????????????????");
						}
					}
					out.write('\n');
				}
				out.write("?");
				for (Entity e : c.getEntities()) {
					out.write(e.encode() + "|");
				}
				out.write("\n");
			}

			out.close();
		} catch (Exception e) {
			return;
		}
	}

	public final World load(final String savename) {
		World nw = new World();

		try {
			FileReader fr = new FileReader(savename + ".sav");
			BufferedReader in = new BufferedReader(fr);

			int mode = 0; // 0: none. 1: chunk header load. 2: data loading. 3:
							// entities loading. (4:set to 0)
			int cx = 0, cy = 0;
			int mx = 0, my = 0;
			int r;
			char c;
			int dao = 0;
			char[] d = new char[16];
			String hd = ""; // header data

			while ((r = in.read()) != -1) {
				c = (char) r;
				if (mode == 0) {
					if (c == '!') {
						mode = 1;
						continue;
					}
				}
				if (mode == 1) {
					if (c == '\n') {
						mode = 2;
						mx = 0;
						my = 0;
						dao = 0;
						try {
							String[] coords = hd.split(","); // !0,0

							if (coords.length != 2) {
								throw new Exception(
										"The coords after '!' must be two, no spaces, sepperated by a comma, integers!");
							}
							cx = Integer.parseInt(coords[0]);
							cy = Integer.parseInt(coords[1]);
						} catch (Exception e) {
							System.out.println(e.getMessage());
							mode = 0;
							hd = "";
							continue;
						} // skip chunk
						hd = "";
						continue;
					}
					hd += c;
				}
				if (mode == 2) {
					if (c == '?') {
						mode = 3;
						continue;
					}
					if (c == '\n') {
						mx++;
						my = 0;
						dao = 0;
						continue;
					}
					d[dao] = c;
					dao++;
					if (dao % 16 == 0) {
						dao = 0;
						// System.out.printf("BLAH[110703 23] x=%d y=%d b=%s\n",
						// mx+cx*Chunk.cs, my+cy*Chunk.cs, Block.decode( //
						// debug
						// new String(d)));
						nw.set(mx + cx * Chunk.cs, my + cy * Chunk.cs,
								Block.decode(new String(d)));
						my++;
						continue;
					}
				}
				if (mode == 3) {
					if (c == '\n') {
						mode = 0;
						continue;
					}
				}
			}

			return nw;
		} catch (Exception e) {
			System.out.println(e.getMessage());
			return null;
		}
	}

	World world;

	public final World getWorld() {
		return world;
	}

	// MG: michal gamrat && maze generation
	int roomsInLine = 200;
	int roomsCount = roomsInLine * roomsInLine;
	Room[][] rooms;
	Stack<Point> recentRooms;
	Point currentRoomCoord;

	public final void createMaze() {

		currentRoomCoord = new Point(0, 0);

		int x, y;

		rooms = new Room[roomsInLine][roomsInLine];

		for (x = 0; x < roomsInLine; x++) {
			for (y = 0; y < roomsInLine; y++) {
				rooms[x][y] = new Room();
			}
		}

		recentRooms = new Stack<Point>();

		this.goToNextRoom();

		while (recentRooms.empty() == false) {
			this.goToNextRoom();
		}

		int roomWidth = View.WIDTH / roomsInLine;
		int roomHeight = View.HEIGHT / roomsInLine;

		int rx, ry, x_offset, y_offset;

		for (y = 0; y < roomsInLine; y++) {
			for (x = 0; x < roomsInLine; x++) {
				// System.out.print(rooms[y][x].toString());

				y_offset = y * roomHeight;
				x_offset = x * roomWidth;

				Room r = rooms[y][x];
				Block b;

				final int linethick = 1;
				
				for (ry = 0; ry < roomHeight; ry++) {
					for (rx = 0; rx < roomWidth; rx++) {
						if (ry < linethick && r.n || ry > roomHeight - linethick && r.s
								|| rx < linethick && r.w || rx > roomWidth - linethick && r.e) {
							b = Block.stone;
							world.set(rx + x_offset, ry + y_offset, b);
						} else {
							// map[rx + x_offset][ry + y_offset] = null;
							world.set(x_offset + rx, y_offset + ry, Block.air);
							// this.set(rx + x_offset, ry + y_offset, null);
						}

					}
				}

			}
			// System.out.println();
		}

	}

	private Set getRooms() {

		Set r = new HashSet();

		if (currentRoomCoord.y > 0
				&& rooms[(int) currentRoomCoord.y - 1][(int) currentRoomCoord.x].visited == false) {
			r.add(1);
		}

		if (currentRoomCoord.y < roomsInLine - 1
				&& rooms[(int) currentRoomCoord.y + 1][(int) currentRoomCoord.x].visited == false) {
			r.add(2);
		}

		if (currentRoomCoord.x > 0
				&& rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x - 1].visited == false) {
			r.add(3);
		}

		if (currentRoomCoord.x < roomsInLine - 1
				&& rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x + 1].visited == false) {
			r.add(4);
		}

		return r;

	}

	private void goToPrevRoom() {

		Point p = recentRooms.pop();

		currentRoomCoord.x = p.x;
		currentRoomCoord.y = p.y;

	}

	private void goToNextRoom() {

		// System.out.println("Current room: "+currentRoom.toString());

		Set r = this.getRooms();

		if (r.isEmpty()) {
			goToPrevRoom();
			return;
		}

		recentRooms.push(new Point(currentRoomCoord.x, currentRoomCoord.y));

		rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x].visited = true;

		Random randomGenerator = new Random();
		int randomInt = randomGenerator.nextInt(r.size());

		int dir = (Integer) r.toArray()[randomInt];

		rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x].visited = true;

		switch (dir) {
		case 1:
			rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x].n = false;
			currentRoomCoord.y -= 1;
			rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x].s = false;
			break;

		case 2:
			rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x].s = false;
			currentRoomCoord.y += 1;
			rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x].n = false;
			break;

		case 3:
			rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x].w = false;
			currentRoomCoord.x -= 1;
			rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x].e = false;
			break;

		case 4:
			rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x].e = false;
			currentRoomCoord.x += 1;
			rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x].w = false;
			break;
		default:
			break;
		}

		rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x].visited = true;

	}

	public final GSnip getImg(final String fortype) {

		try {
			BufferedImage img = ImageIO.read(new File(new File(".")
					.getCanonicalPath() + "/" + fortype + ".bmp"));
		} catch (IOException e) {
			System.out.println("[model.getImg(..)] Image not found for type \""
					+ fortype + "\"");
			return null;
		}

		// //////////////////////////////////////////////////
		// //////////////////////////////////////////////////
		// //////////////////////////////////////////////////
		// //////////////////////////////////////////////////
		// //////////////yay/////////////////////////////////
		// //////////////////////////////////////////////////
		// //////////////////////////////////////////////////
		// //////////////////////////////////////////////////
		// //////////////////////////////////////////////////

		return null;
	}

	// PROCESSING
	// huh :o? @see World / Chunk / Entity for raytracing

	// PROPERTIFY
	private HashMap<String, Object> propertifies;

	public final void propset(final String key, final Object value) {
		propertifies.put(key, value);
	}

	public final Object prop(final String key) {
		return propertifies.get(key);
	}

	// VARIOUS RESSOURCES

	private WindowManager wm;

	public final WindowManager getWM() {
		return wm;
	}

	// Map scrolling
	int offx, offy; // how many BLOCKS(Model.zoom). The post-comma part is
					// pixels (may be buggy of course)
	int noffx, noffy;
	public static final int zoom = 10; // blocksize (if zoom==1 then offx/y
										// could be int)

	public final synchronized void scrollToCentrify(final int x, final int y) {
		noffx = (int) (x - (View.WIDTH / zoom / 2)); // TODO centrify really
		noffy = (int) (y - (View.HEIGHT / zoom / 2));
	}

	public final synchronized void scrollTo(final int x, final int y) {
		noffx = x;
		noffy = y;
	}

	public final void setScrolls() {
		this.offx = noffx;
		this.offy = noffy;
	}

	public final int scrollX() {
		return offx;
	}

	public final int scrollY() {
		return offy;
	}

	// Compiz
	private List<Snippet> compizsnippets;
	private Compiz compiz;

	// TODO buggy (lazy initialisation of compiz)
	public @Deprecated
	final Compiz compizEnsure(final Compiz cpz) {
		if (this.compiz != null) {
			this.compiz = cpz;
		}
		return compiz;
	}

	public @Deprecated
	final Compiz compizEnsure(final GraphicsEngine ge) {
		if (this.compiz == null) {
			return (this.compiz = new Compiz(ge));
		}
		return compiz;
	}

	/*
	 * If you arent sure if compiz yet has been created. you might use
	 * compizEnsure (the one taking GraphicsEngine). They're very handy. [[[>
	 * WARNING: as of my new Standard v0.11R10IN this is deprecated. init*() are
	 * now in use! <]]]
	 * 
	 * @return Compiz. NULL otherwise. (otherwise use compizEnsure(..));
	 */
	public final Compiz compizGet() {
		return compiz;
	}

	public final List<Snippet> compizSnippetsGet() {
		return compizsnippets;
	}

	public @Deprecated
	final void compizSnippetsAdd(final Snippet snip) {
		compizsnippets.add(snip);
	}

	public final void compizSnippetsRemove(final Snippet snip) {
		compizsnippets.remove(snip);
	}

	// pausing
	private boolean paused = false;

	public final void pause(final boolean p) {
		this.paused = p;
	}

	public final boolean pause() {
		return paused;
	}

	// data
	private EntityPlayer player;

	public final EntityPlayer getPlayer() {
		return player;
	}

	// terminal (WIP)
	private Vector<String> termtext;
	private boolean termextend = false; // fullscreen terminal

	public final Vector<String> termGetText() {
		return termtext;
	}

	public final void termPost(final String text) {
		termtext.add(text);
		if (termtext.size() < 6) {
			return;
		}
		termtext.remove(0);
	}

	public final String termLast(final int neg) {
		int npos = termtext.size() - neg - 1;

		if (npos < 0 || npos >= termtext.size()) {
			return null;
		}
		return termtext.get(npos);
	}

	public final boolean termExtend() {
		return termextend;
	}

	public final void termExtend(final boolean termextend) {
		this.termextend = termextend;
	}

	// form((forms are deprecated. Windows/containers)) managment (see getWM()
	// and WindowManager in view.gui )
	private boolean useforms = false;

	public final boolean useForms() {
		return useforms;
	}

	public final void useForms(final boolean useforms) {
		this.useforms = useforms;
	}

}
