package model;

import java.awt.image.BufferedImage;
import java.io.*;
import java.util.HashMap;
import java.util.List;
import java.util.LinkedList;
import java.util.Vector;

import javax.imageio.ImageIO;

import view.Compiz;
import view.GraphicsEngine;
import view.Snippet;
import view.GSnip;
import view.View;
import view.gui.WindowManager;

import java.io.File;
import java.io.IOException;
import java.util.HashSet;
import java.util.Random;
import java.util.Set;
import java.util.Stack;

public class Model {
	
	public Model() {
		propertifies     = new HashMap<String, Object>(100);
		compizsnippets   = new LinkedList<Snippet>();
		wm               = new WindowManager();
		world            = new World();
		player           = new EntityPlayer(world, 3, 3);
		System.out.printf("CORE INIT [model.Model this(..) news] (EntityPlayer)player.eid = %d\n", player.getEID());
		
		BufferedImage img = null;
		try {
			img = ImageIO.read(new File(new File(".").getCanonicalPath() + "/worldfix_map_110529.bmp"));
			for (int x=0; x<img.getWidth(); x++) {
				for (int y=0; y<img.getHeight(); y++) {
					if (img.getRGB(x, y) != -1) world.set(x, y, Block.stone);
					else                        world.set(x, y, Block.air);
				}
			}
		} catch (Exception ex) { System.out.println("Model.this(): couldnt load map"); }
		
		termtext = new Vector<String>();
	}
	
	/**e.g. "bobs save" ((--> "./bobs save.sav") */
	public void save(String savename) {
		try {/* serialisation: sux
			FileOutputStream fout = new FileOutputStream(savename+".sav");
			ObjectOutputStream oos = new ObjectOutputStream(fout);
			oos.writeObject(this);
			oos.close();*/
			
			FileWriter fw = new FileWriter(savename+".sav");
			BufferedWriter out = new BufferedWriter(fw);
			
			for (int x=0; x<View.WIDTH; x++) {
				for (int y=0; y<View.HEIGHT; y++) {
					Block b = world.get(x,y);
					if (b==null || b == Block.air) { out.write(" "); continue; }
					System.out.println(b.toString());
					out.write(b.toString());
				}
				out.write('\n');
			}
			
			// end of map data
			out.write('\n');
			out.write('\n');
			
			// todo: entities
			/* 110629: commented out: minecraftification
			for (Block b : enten) {
				out.write(p.toString());
			}
			*/
			
			out.write("\n");
			
			out.close();
		} catch (Exception e) { return; }
	}
	
	World world;
	public World getWorld() { return world; }
	
	// MG: michal gamrat && maze generation
	int roomsInLine = 19;
	int roomsCount = roomsInLine * roomsInLine;
	Room[][] rooms;
	Stack<Point> recentRooms;
	Point currentRoomCoord;

	public void createMaze() {

		currentRoomCoord = new Point(0, 0);

		int x, y;

		rooms = new Room[roomsInLine][roomsInLine];

		for(x=0; x<roomsInLine; x++){
			for(y=0; y<roomsInLine; y++){
				rooms[x][y] = new Room();    
			}   
		}


		recentRooms = new Stack<Point>();

		this.goToNextRoom();

		while(recentRooms.empty() == false){
			this.goToNextRoom();
		} 

		int roomWidth = View.WIDTH / roomsInLine;
		int roomHeight = View.HEIGHT / roomsInLine;

		int rx, ry, x_offset, y_offset;

		for(y=0; y<roomsInLine; y++){
			for(x=0; x<roomsInLine; x++){
				//System.out.print(rooms[y][x].toString());

				y_offset = y * roomHeight;
				x_offset = x * roomWidth;

				Room r = rooms[y][x];
				Block b;

				for(ry=0; ry<roomHeight; ry++){
					for(rx=0; rx<roomWidth; rx++){
						if(ry < 3 && r.n || ry > roomHeight-3 && r.s || rx<3 && r.w || rx>roomWidth-3 && r.e ){      
							b = Block.stone;
							world.set(rx + x_offset, ry + y_offset, b);
						}else{       
							//map[rx + x_offset][ry + y_offset] = null;
							world.set(x_offset+rx, y_offset+ry, Block.air);
							//this.set(rx + x_offset, ry + y_offset, null);      
						}

					}
				}    

			}
			//System.out.println();
		}


	}
	private Set getRooms(){

		Set r = new HashSet();

		if(currentRoomCoord.y > 0 && rooms[(int) currentRoomCoord.y - 1][(int) currentRoomCoord.x].visited == false){
			r.add(1);
		}

		if(currentRoomCoord.y < roomsInLine - 1 && rooms[(int) currentRoomCoord.y + 1][(int) currentRoomCoord.x].visited == false){
			r.add(2);
		}

		if(currentRoomCoord.x > 0 && rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x - 1].visited == false){
			r.add(3);
		}

		if(currentRoomCoord.x < roomsInLine - 1 && rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x + 1].visited == false){
			r.add(4);
		}

		return r;

	}
	private void goToPrevRoom(){

		Point p = recentRooms.pop();
		currentRoomCoord.x = p.x;
		currentRoomCoord.y = p.y;

	}
	private void goToNextRoom(){

		//System.out.println("Current room: "+currentRoom.toString());

		Set r = this.getRooms();

		if(r.isEmpty()){
			goToPrevRoom();
			return;
		}

		recentRooms.push(new Point(currentRoomCoord.x, currentRoomCoord.y));

		rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x].visited = true;

		Random randomGenerator = new Random();  
		int randomInt = randomGenerator.nextInt(r.size());  

		int dir = (Integer)r.toArray()[randomInt];

		rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x].visited = true;

		switch(dir){
		case 1:   
			rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x].n = false;
			currentRoomCoord.y -= 1;
			rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x].s = false;
			break;
		case 2:
			rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x].s = false;
			currentRoomCoord.y += 1;
			rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x].n = false;
			break;
		case 3:
			rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x].w = false;
			currentRoomCoord.x -= 1;
			rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x].e = false;
			break;
		case 4:
			rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x].e = false;
			currentRoomCoord.x += 1;
			rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x].w = false;
			break;
		}

		rooms[(int) currentRoomCoord.y][(int) currentRoomCoord.x].visited = true;

	}
	
	public GSnip getImg(String fortype) {
		
		try {
			BufferedImage img = ImageIO.read(new File(new File(".").getCanonicalPath()+"/"+fortype+".bmp"));
		} catch (IOException e) {
			System.out.println("[model.getImg(..)] Image not found for type \""+fortype+"\"");
			return null;
		}

		////////////////////////////////////////////////////
		////////////////////////////////////////////////////
		////////////////////////////////////////////////////
		////////////////////////////////////////////////////
		////////////////yay/////////////////////////////////
		////////////////////////////////////////////////////
		////////////////////////////////////////////////////
		////////////////////////////////////////////////////
		////////////////////////////////////////////////////
		
		return null;
	}
	
	// PROCESSING
	// huh :o? @see World / Chunk / Entity
	
	// PROPERTIFY
	private HashMap<String, Object> propertifies;
	public void propset(String key, Object value) { propertifies.put(key, value); }
	public Object prop(String key) { return propertifies.get(key); }
	
	// VARIOUS RESSOURCES
	
	private WindowManager wm;
	public WindowManager getWM() { return wm; }
	
	// Map scrolling
	double offx, offy; // how many BLOCKS(Model.zoom). The post-comma part is pixels (may be buggy of course)
	double noffx, noffy;
	public final static int zoom = 2; // blocksize (if zoom==1 then offx/y could be int)
	public synchronized void scrollToCentrify(double x, double y) {
		noffx = (int)(x-(View.WIDTH /zoom/2)); // TODO centrify realy
		noffy = (int)(y-(View.HEIGHT/zoom/2));
	}
	public synchronized void scrollTo(double x, double y) {
		noffx = x;
		noffy = y;
	}
	public void setScrolls() { this.offx = noffx; this.offy = noffy; }
	public double scrollX() { return offx; }
	public double scrollY() { return offy; }
	
	// Compiz
	private List<Snippet> compizsnippets;
	private Compiz compiz;
	
	// TODO buggy (lazy initialisation of compiz)
	public @Deprecated Compiz compizEnsure(Compiz cpz) {
		if (this.compiz != null) this.compiz = cpz;
		return compiz;
	}
	public @Deprecated Compiz compizEnsure(GraphicsEngine ge) {
		if (this.compiz == null) return (this.compiz = new Compiz(ge));
		return compiz;
	}

	/*
	 * If you arent sure if compiz yet has been created. you might use compizEnsure (the one taking GraphicsEngine).
	 * They're very handy. [[[> WARNING: as of my new Standard v0.11R10IN this is deprecated. init*() are now in use! <]]] 
	 * @return Compiz. NULL otherwise. (otherwise use compizEnsure(..));
	 */
	public Compiz compizGet() { return compiz; }
	public List<Snippet> compizSnippetsGet() {
		return compizsnippets;
	}
	public @Deprecated void compizSnippetsAdd(Snippet snip) {
		compizsnippets.add(snip);
	}
	public void compizSnippetsRemove(Snippet snip) {
		compizsnippets.remove(snip);
	}
	
	// pausing
	private boolean paused = false;
	
	public void pause(boolean p) { this.paused = p; }
	public boolean pause() { return paused; }
	
	// data
	private EntityPlayer player; // :(
	
	/* 110629: commented out: switching to Worlds
	public synchronized Pixie get(int x, int y) {
		if (x<0 || x>=map.length || y<0 || y>=map[x].length) { return null; } // XXX buggy
		//if (map[x][y] == null) map[x][y] = new Pixie(0);
		return map[x][y];
	}
	public synchronized void set(int x, int y, Pixie pix) {
		if (x<0 || x>=map.length || y<0 || y>=map[x].length) return;
		setmap[x][y] = pix;
	}
	public synchronized void set(float x, float y, Pixie pix) {
		if (x<0 || x>=map.length || y<0 || y>=map[Math.round(x)].length) return;
		setmap[Math.round(x)][Math.round(y)] = pix;
		pix.x = x;
		pix.y = y;
	}
	public synchronized void setMap() {
		for (int x=0; x<map.length; x++) {
			for (int y=0; y<map[x].length; y++) {
				map[x][y] = setmap[x][y];
			}
		}
	} */
	public EntityPlayer getPlayer() { return player; }
	
	// terminal (WIP)
	private Vector<String> termtext;
	private boolean termextend = false; // fullscreen terminal
	
	public Vector<String> termGetText() { return termtext; }
	public void termPost(String text) { termtext.add(text); if (termtext.size() < 6) return; termtext.remove(0); }
	public String termLast(int neg) { int npos=termtext.size()-neg-1; if (npos<0 || npos>=termtext.size()) return null; return termtext.get(npos); }
	public boolean termExtend() { return termextend; }
	public void termExtend(boolean termextend) { this.termextend = termextend; }
	
	// form((forms are deprecated. Windows/containers)) managment (see getWM() and WindowManager in view.gui )
	private boolean useforms = false;

	public boolean useForms() {
		return useforms;
	}
	public void useForms(boolean useforms) {
		this.useforms = useforms;
	}
	
}
