package model;

import java.util.LinkedList;
import java.util.List;

import mutil.Undone;

/*
 * COORD MODIFIERS:
 * absolute / relative
 * normal / chunkish
 * 
 * a	absolute: from world's zero
 * r	relative: from current chunk's zero
 * 
 * -	normal: in block coords context
 * ch	chunkish: in chunk coords context
 * `-> (chunkish*cs) == normal
 * `-> (Math.floor(normal/cs) == chunkish)
 * 
 */

public class World {
	private static final int cs = Chunk.cs;

	List<Chunk> chunks; // all the chunks
	private Entity[] entities; // global entities storer
	private List<Entity> entitiesl; // their list.
	long lasttime;

	public World() {
		chunks = new LinkedList<Chunk>();
		entities = new Entity[65536];
		entitiesl = new LinkedList<Entity>();
		lasttime = System.currentTimeMillis();
		tick();
	}

	public List<Entity> getEntities() {
		return entitiesl;
	}

	public List<Chunk> getChunks() {
		return chunks;
	}

	public @Deprecated
	void makeChunk(int chx, int chy) {
		Chunk c = new Chunk(this, chx, chy);

		chunks.add(c);
	}

	/** ret[1]: wall. ret[0]: last okay position */
	public @Undone
	BlockX[] raytrace(double x1, double y1, double x2, double y2) {

		BlockX[] ret = new BlockX[2];

		ret[0] = new BlockX(this.get((int) x1, (int) y1), x1, x1);
		ret[1] = new BlockX(this.get((int) x2, (int) y2), x2, x2);

		if (!(ret[1].getOrig().solid)) {
			return null;
		}
		return ret;

		// float len = (float) Math.sqrt(Math.pow(x2 - x1, 2) + Math.pow(y2 -
		// y1, 2));
		// if ( len == 0 ) {
		// //TODO: Make this function stop if this happens.
		// return null;
		// }
		//
		// double addx = (x2 - x1)/len;
		// double addy = (y2 - y1)/len;
		//
		// while (((int) x1 != (int) x2 || (int) y1 != (int) y2) && (addx < 0 ?
		// x1 >= x2 : x1 <= x2) && (addy < 0 ? y1 >= y2 : y1 <= y2)) {
		// x1 += addx;
		// y1 += addy;
		//
		// Block b = get((int)Math.round(x1), (int)Math.round(y1));
		// if (b == null || b == Block.air) continue;
		// //more emptiness conditions
		//
		// BlockX[] ret = new BlockX[2];
		//
		// BlockX prev = new BlockX(Block.air, x1-addx, y1-addy);
		// BlockX curr = new BlockX(null);
		// ret[1] = new BlockX(b); // the wall
		// if (prev != null) {
		// ret[0] = new BlockX(null); // TODO ask evan
		// } else {
		// ret[0] = new BlockX(Block.air, x1-addx, y1-addy); // yay
		// }
		// return ret;
		// }
		//
		// return null;

	}

	public long entityallocate(Entity ente) {
		synchronized (entitiesl) {
			for (int i = 0; i < entities.length; i++) {
				if (entities[i] == null) {
					entities[i] = ente;
					System.out.printf("WORLD [Entity %d] Allocated EID for %s\n",
							i, ente);
					entitiesl.add(ente);
					return i;
				}
			}
			System.out.printf("WORLD [Entity -1] Failed to allocate EID for %s\n",
					ente);
		}
		return -1;
	}

	public void entitykill(long eid) {
		synchronized(entitiesl) {
			if (eid >= 0 && eid < entities.length) {
				Entity e = entities[(int) eid];

				if (e == null) {
					System.out
					.printf("WORLD [Entity %d] Failed to kill Entity by EID (Slot already null)\n",
							eid);
					return;
				}

				entities[(int) eid] = null;
				Chunk c;
				entitiesl.remove(e);

				(c = e.getChunk()).entityrem(eid);
				System.out.printf("WORLD [Entity %d] Killed %s from %s\n", e, c);
			}
			System.out
			.printf("WORLD [Entity %d] Failed to kill Entity by EID (Out of range)\n",
					eid);
		}
	}

	public void entitiesUpdate(double delta) {
		synchronized(entitiesl) {
			for (int i=0; i<entitiesl.size(); i++) {
				Entity e = entitiesl.get(i);
				if (e != null) {
					e.update(delta);
				}
			}
		}

	}

	public void tick() {
		long nowtime = System.currentTimeMillis();

		entitiesUpdate(((double) nowtime - (double) lasttime) / 1000d);
		lasttime = nowtime;
	}

	public Block get(int x, int y) {
		Chunk c = getChunkAt(x, y);

		if (c != null) {
			return c.geta(x, y);
		}
		return Block.air;
	}

	public void set(int x, int y, Block datatype) {
		Chunk c = ensureChunkAt(x, y);

		c.seta(x, y, datatype);
	}

	public Chunk ensureChunkAt(int x, int y) {
		// for (Chunk c : chunks) if (x>=c.x*cs && y>=c.y*cs && x<(c.x+1)*cs &&
		// y<(c.y+1)*cs) return c;
		Chunk c = getChunkAt(x, y);

		if (c != null) {
			return c;
		}

		c = new Chunk(this, (int) Math.floor(x / cs), (int) Math.floor(y / cs));
		chunks.add(c);
		return c;
	}

	public Chunk getChunkAt(int x, int y) {
		for (Chunk c : chunks) {
			if (x >= c.x * cs && y >= c.y * cs && x < (c.x + 1) * cs
					&& y < (c.y + 1) * cs) {
				return c;
			}
		}
		return null;
	}

	public Chunk getChunk(int chx, int chy) {
		for (Chunk c : chunks) {
			if (chx == c.x && chy == c.y) {
				return c;
			}
		}
		return null;
	}
}
