package model.pixies;

import java.awt.Color;

import model.Model;

import view.GSnip;
import view.GraphicsEngine;

/** Overwritables:
 *  render
 *  update
 *  getName getLongName
 *  getType getLongType
 *  move
 *  
 *  static type2str str2type
 */
public class Pixie {
	public static final String myname = "Pixie"; // overwrite this.
	public float x, y; // for finer calculations
	protected Type type;
	
	public enum Type {
		ERROR,		// error	should be really just used for error codes or anything like that.
		NULL,		// null		instead of setting the pixie to null. or to make it unusable on purpose.
		STD,		// Pixie
		COORDS,		// Pixie	being _used_ from raytracepixies to tell one the coords, if theres no pixie there. (Pixie itself)
		
		Actor,		// Actor	--> {Player, Monster}
		Material,	// Material	--> {Stone, Dirt}
		Item,		// Item 	e.g. in inventory TODO
	}
	
	// statics
	public static String type2str(Type type) {
		switch (type) {
		case NULL: return "NULL";
		case STD: return "STD";
		case COORDS: return "COORDS";
		case Actor: return "Actor";
		case Material: return "Material";
		case Item: return "Item";
		}
		
		// default:
		return "ERROR";
	}
	public static Type str2type(String str) {
		return Type.ERROR; // TODO
	}
	public static final String getShortIdentifier(Pixie of) {
		if (of == null || of.getT()==Type.NULL) return " ";
		String t = of.getName();
		System.out.println(of.getLongType());
		//if (t.equals("STD")) return "#";
		//if (t.equals("AIR")) return "0";
		
		// t = t.substring(t.length()-5);
		
		if (t.equals("Bullet")) return "b";
		if (t.equals("Acid")) return "a";
		if (t.equals("Actor")) return "A";
		if (t.equals("Dirt")) return "d";
		if (t.equals("Item")) return "I";
		if (t.equals("Laser")) return "l";
		if (t.equals("Material")) return "M";
		if (t.equals("Monster")) return "X"; // FAIL FIXME
		if (t.equals("Player")) return "P";
		if (t.equals("Stone")) return "s";
		if (t.equals("Spawner")) return "S"; // FAIL FIXME
		if (t.equals("Thing")) return "T";
		if (t.equals("Water")) return "w";
		
		return "?"; // wut
	}
	
	public String toString() {
		String ret = "#";
		
		return ret;
	}
	
	public static final int getValue(Pixie wut) {
		//switch (wut)
		return -1;
	}
	
	public Pixie(Pixie.Type type) {
		this.type = type;
		// no super. this is hyper
	}

	public void update(Model m, float delta) {
		m.set(getX(), getY(), this); // else we'd get removed
	}
	public void render(GraphicsEngine ge) {
		// Pixie.STD
		ge.set(getX(), getY(), Color.gray); // ORANGEnew Color(130, 170, 220)
	}
	public Color getColor() {
		return Color.gray;
	}
	
	/**Move function with integrated high-end collision checking, aswell
	 * as relative coords and insane auto updating maps!
	 * Only 2.99$!
	 * 
	 * @param relx Relative amount of pixels in X
	 * @param rely Relative amount of pixels in Y
	 * @param m Model.
	 * @return 0: no collision
	 */
	public int move(float relx, float rely, Model m) {
		
		
		Pixie[] ps = m.raytracePixies(x, y, relx+x, rely+y);
		
		if (ps == null) { // free space
			x+=relx;
			y+=rely;
		} else { // collision
			
		}
		
		
		return 0;
	}
	
	public int getX() { return (int)x; }
	public int getY() { return (int)y; }
	
	public final Pixie.Type getT() { return type; } // in case to check for Pixie.Type.NULL or anything else
	public String getType() { return type2str(type); }
	public String getLongType() { return type2str(type); }
	public static String getName() { return myname; }
	public String getLongName() { return getName(); }
	
	public GSnip getImg() {
		
		//Color[][] cmap = new Color[24][24];
		
		GSnip gs = new GSnip(new Color[][]
		{
				{new Color(200,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
				{new Color(000,000,000),new Color(000,200,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
				{new Color(000,000,000),new Color(000,000,000),new Color(000,000,200),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
				{new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
				{new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
				{new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
				{new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
				{new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
				{new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
				{new Color(000,000,000),new Color(000,000,000),new Color(200,255,050),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
				{new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
				{new Color(000,000,000),new Color(000,000,000),new Color(128,220,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
				{new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
				{new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
				{new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
				{new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
				{new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
				{new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
				{new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
				{new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
				{new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
				{new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
				{new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000),new Color(000,000,000)},
		});
		
		return gs;
	}
}








       








