package model.pixies;

import java.awt.Color;

import model.Inventory;
import model.Model;
import model.Point;

import view.GraphicsEngine;

/** leav of the Pixies tree */
public class Player extends Actor {
	public static final String myname = "Player";
	private Inventory inv, palette, armor;
	
	private float speed;
	private float dir;
	
	private int level;
	private float hp;
	private int exp, maxexp;
	
	public Player() {
		super(Actor.Type.Player);
		// this class is somewhat final
		
		levelup(1); // fakeish
		
		inv      = new Inventory(Inventory.Type.Player,        8*3); // 8x 3y field
		palette  = new Inventory(Inventory.Type.PlayerPalette, 8*1); // one row, 8 columens
		armor    = new Inventory(Inventory.Type.PlayerArmor,   1*4); // 4 slots vertical
	}
	
	public int getLevel() { return level; }
	public float getHP() { return hp; }
	public int getEXP() { return exp; }
	public void expGain(int exp) {
		exp += exp;
		if (exp >= maxexp) levelup(level+1);
	}
	public void levelup(int levelnew) {
		maxexp = (int)(level*Math.PI+2f)+10;
		exp = 0;
		level++;
	}
	
	public static String getName() { return myname; }
	public String getLongName() { return super.getLongName() + "." + getName(); }
	public String getType() { return type2str(type); }
	public String getLongType() { return super.getLongType() + "." + type2str(type); }
	public String getShortIdentifier() { return "A"; }
	
	public String toString() {
		String ret = "$";
		
		return ret;
	}
	
	public Inventory getInv() { return inv; }
	public Inventory getInvPalette() { return palette; }
	public Inventory getInvArmor() { return armor; }
	
	@Override
	public int move(float relx, float rely, Model m) {
		float tspeed = 1;
		float tdir = (float)(Math.atan2(rely, relx));
		
		speed = tspeed;
		dir = tdir;
		//dir = (dir*(speed/2) + tdir*tspeed)/2;
		
		/*
		Point[] ps = m.raytracePoints(x,y, x+relx, y+rely);
		float prevx=x, prevy=y;
		if (ps == null) {
			this.x += relx;
			this.y += rely;
		} else {
			if (ps[0] != null) {
				this.x = ps[0].x;
				this.y = ps[0].y;
			}
			m.termPost("model.Player.move(..): Collision detected! ("+x+","+y+")");
		}
		m.set((int)prevx, (int)prevy, null);
		*/

		
		return 0;
	}
	
	/**Pick something up. get it anyhow into inventory.
	 * @return 0: success. -1: inventory full
	 */
	public int pickup(Pixie wut) {
		
		// TODO
		
		return 0;
	}
	
	/**Shoots/throws anything
	 * @param tox Destination X
	 * @param toy Destination Y
	 * @param speed Speed. 1.0f = one pixel per frame
	 * @param dir The direction; in RADIANS
	 * @param projectile What projectile. (Take Bullet here)
	 */
	public void shoot(float tox, float toy, float speed, float dir, Thing projectile) { // TODO: Add "Item bywhat" (gun, bow, whatever)
		// outgoing of the players position
		projectile.x = x;
		projectile.y = y;
		projectile.speed = speed;
		projectile.dir = dir;
	}
	
	@Override
	public void render(GraphicsEngine ge) {
		for (int mx=-30; mx<30; mx++) {
			for (int my=-30; my<30; my++) {
				float dist = (float)Math.sqrt(Math.pow(mx,2) + Math.pow(my,2));
				if (dist <= 0) dist = -dist;
				dist = (10-dist);
				
				//dist = dist/10f; // now we have from 1.0f-0.0f (strength)
				//dist = dist-(float)Math.sqrt(dist);
				//dist = dist / (dist*dist);
				//dist = (float) Math.cbrt(dist/10); // try tan. LOL!
				dist = (float) Math.tan((dist+7000)/100)-1.10f; // quite good! ( TODO )
				// dist = (float) Math.tan((dist+7000)/100)/2.00f-0.45f; // excellent. not good on macbook. weird
				// dist = (float) Math.cbrt(dist); // not good
				if (dist <= 0) dist = 0;
				
				
				Color c = ge.get(Math.round(x-mx), Math.round(y-my));
				float r=(c.getRed()   /255f);
				float g=(c.getGreen() /255f);
				float b=(c.getBlue()  /255f);
				//r = dist*1+(1-dist)*r;
				//g = dist*1+(1-dist)*g;
				//b = dist*1+(1-dist)*b;
				
				
				r += dist;
				g += dist;
				b += dist;
				
				if (r>1) r=1; else if (r<0) r=0;
				if (g>1) g=1; else if (g<0) g=0;
				if (b>1) b=1; else if (b<0) b=0;
				
				ge.set(Math.round(x-mx), Math.round(y-my), new Color(r, g, b));
				
			}
		}
		
		ge.set(Math.round(x), Math.round(y), Color.white);
	}
	
	@Override
	public void update(Model m, float delta) {
		//super.update(m, delta);
		
		float px=x, py=y;
		
		x += (float)(Math.cos(dir)*speed);
		y += (float)(Math.sin(dir)*speed);
		
		Point[] ps = m.raytracePoints(px,py, x,y);
		if (ps == null) {
			speed /= 1.6;
		} else {
			// collsion
			x=ps[0].x;
			y=ps[0].y;
			speed = -(speed/2);
			m.termPost("model.Player.move(..): Collision detected! ("+x+","+y+")");
		}
		m.set((int)x, (int)y, null);
		
		m.scrollTo(x, y);
	}
	
}





