package view;

import java.awt.Color;

import model.*;


public abstract class RenderEntity {
	private static final int zoom = Model.zoom;
	// private static final int cs = Chunk.cs;
	private static Model model;

	public static void initModel(Model model) {
		RenderEntity.model = model;
	}

	private static int offx() {
		if (model == null) {
			return 0;
		}
		return model.scrollX();
	}

	private static int offy() {
		if (model == null) {
			return 0;
		}
		return model.scrollY();
	}

	public static void render(Entity wut, GraphicsEngine ge) {
		if      (wut instanceof EntityPlayer) renderPlayer(wut, ge);
		else if (wut instanceof EntityAnimation) RenderAnimation.render((EntityAnimation)wut, ge);
		else if (wut instanceof EntityProjectile) renderProjectile(wut, ge);
		else if (wut instanceof EntityMonster) {
			if      (wut instanceof EntityMonsterLiver) renderMonsterLiver(wut, ge);
			else if (wut instanceof EntityMonsterSadfacer) renderMonsterSadfacer(wut, ge);
			else if (wut instanceof EntityMonsterCokeDrinker) renderMonsterCokeDrinker(wut, ge);
			else if (wut instanceof EntityMonsterApple) renderMonsterApple(wut, ge);
			else if (wut instanceof EntityMonsterMicrosoft) renderMonsterMicrosoft(wut, ge);
			else renderMonster(wut, ge);
		}
		else if (wut instanceof EntitySpawner) renderSpawner(wut, ge);
	}

	public static void renderSpawner(Entity wut, GraphicsEngine ge) {
		simpleRender(wut, ge, new Color((float)Math.random(), (float)Math.random(), (float)Math.random(), (float)Math.random()));
	}
	
	private static void renderMonsterMicrosoft(Entity wut, GraphicsEngine ge) {
		//simpleRender(wut, ge, Color.DARK_GRAY.darker().darker());
		ge.text((int) ((wut.getX() - offx()) * zoom), (int) ((wut.getY() - offy()) * zoom)-2, ""+(char)128, Color.red.brighter(), 2);
	}
	
	private static void renderMonsterApple(Entity wut, GraphicsEngine ge) {
		simpleRender(wut, ge, Color.green);
	}
	
	private static void renderMonsterCokeDrinker(Entity wut, GraphicsEngine ge) {
		simpleRender(wut, ge, new Color(150,60,0,170));
	}
	
	private static void renderMonsterSadfacer(Entity wut, GraphicsEngine ge) {
		simpleRender(wut, ge, Color.gray);
		ge.text((int) ((wut.getX() - offx()) * zoom)+3, (int) ((wut.getY() - offy()) * zoom)+2, ""+(char)129, Color.red.darker(), 2);
	}
	
	private static void renderMonsterLiver(Entity wut, GraphicsEngine ge) {
		simpleRender(wut, ge, new Color(255,230,230,255));
	}
	
	private static void renderMonster(Entity wut, GraphicsEngine ge) {
		simpleRender(wut, ge, Color.red);
	}

	private static void renderPlayer(Entity wut, GraphicsEngine ge) {
		EntityPlayer wutt = (EntityPlayer)wut;
		simpleRender(wut, ge, new Color(1f,1f,1f, (float)wutt.getHP()/(float)wutt.getMaxHP() )); 
	}

	private static void renderProjectile(Entity wut, GraphicsEngine ge) {
		simpleRender(wut, ge, Color.yellow);
	}

	private static void simpleRender(Entity wut, GraphicsEngine ge, Color c) {
		simpleFill(ge, (int) ((wut.getX() - offx()) * zoom),
				(int) ((wut.getY() - offy()) * zoom), c);
	}
	private static void simpleFill(GraphicsEngine ge, int x, int y, Color c) {
		simpleFill(ge, x, y, c, zoom, zoom);
	}
	private static void simpleFill(GraphicsEngine ge, int x, int y, Color c,
			int xs, int ys) {
		for (int mx = 0; mx < xs; mx++) {
			for (int my = 0; my < ys; my++) {
				ge.seta(mx + x, my + y, c);
			}
		}
	}
}

/*
 * 110703: before coord fix. package view;
 * 
 * import java.awt.Color;
 * 
 * import model.*;
 * 
 * /**The xoff/yoff should always the model.scrollX/Y and it should render fine.
 * 
 * 
 * /
 * 
 * public abstract class RenderEntity { private static final int zoom =
 * Model.zoom; // private static final int cs = Chunk.cs; private static Model
 * model; public static void initModel(Model model) { RenderEntity.model =
 * model; } private static int offx() { if (model == null) return 0; return
 * model.scrollX(); } private static int offy() { if (model == null) return 0;
 * return model.scrollY(); }
 * 
 * public static void render(Entity wut, GraphicsEngine ge, int xoff, int yoff)
 * { if (wut instanceof EntityPlayer) renderPlayer (wut, ge,xoff,yoff); else if
 * (wut instanceof EntityProjectile) renderProjectile (wut, ge,xoff,yoff); }
 * 
 * private static void renderMonster(Entity wut, GraphicsEngine ge) {
 * 
 * } private static void renderPlayer(Entity wut, GraphicsEngine ge,int x,int y)
 * { simpleFill(ge,x,y, Color.white); } private static void
 * renderProjectile(Entity wut, GraphicsEngine ge,int x,int y) {
 * simpleFill(ge,x,y, Color.yellow); }
 * 
 * private static void simpleFill(GraphicsEngine ge, int x, int y, Color c, int
 * xs, int ys) { for (int mx=0; mx<xs; mx++) { for (int my=0; my<ys; my++) {
 * ge.set(mx+x, my+y, c); } } } private static void simpleFill(GraphicsEngine
 * ge, int x, int y, Color c) { simpleFill(ge, x, y, c, zoom, zoom); } }
 */

