package view.gui;

import java.awt.Color;

import view.GSnip;
import view.GraphicsEngine;

public class GSnipComponent extends Component {

	public Color[][] map;

	public GSnipComponent(int xsize, int ysize) {
		map = new Color[xsize][ysize];
	}

	public int xlen() {
		return map.length;
	}

	public int ylen() {
		return map[0].length;
	}

	@Override
	public GSnip redraw() {
		return new GSnip(map);
	}

	public void drawonto(GSnip onto, int x, int y) {
		for (int mx = 0; mx < xlen(); mx++) {
			for (int my = 0; my < ylen(); my++) {
				onto.set(
						x + mx,
						y + my,
						GraphicsEngine.blenda(onto.get(x + mx, x + my),
								this.get(mx, my)));
			}
		}
	}

	public void drawonto(GSnip onto, int x, int y, int maxx, int maxy) {
		for (int mx = 0; mx < maxx; mx++) {
			for (int my = 0; my < maxy; my++) {
				onto.set(
						x + mx,
						y + my,
						GraphicsEngine.blenda(onto.get(x + mx, x + my),
								this.get(mx, my)));
			}
		}
	}

	public void drawfrom(GSnip source, int x, int y) {
		for (int mx = 0; mx < source.xlen(); mx++) {
			for (int my = 0; my < source.ylen(); my++) {
				this.set(
						mx + x,
						my + y,
						GraphicsEngine.blenda(this.get(x + mx, my + y),
								source.get(mx, my)));
			}
		}
	}

	public void drawfrom(GSnip source, int x, int y, int maxx, int maxy) {
		for (int mx = 0; mx < maxx; mx++) {
			for (int my = 0; my < maxy; my++) {
				this.set(
						x + mx,
						my + y,
						GraphicsEngine.blenda(this.get(mx + x, my + y),
								source.get(mx, my)));
			}
		}
	}

	public void render(GraphicsEngine ge, int x, int y) {
		for (int mx = 0; mx < xlen(); mx++) {
			for (int my = 0; my < ylen(); my++) {
				ge.set(mx + x, my + y, GraphicsEngine.blend(
						ge.get(mx + x, my + y), get(mx, my)));
			}
		}
	}

	public void render(GraphicsEngine ge, int x, int y, int maxx, int maxy) {
		for (int mx = 0; mx < maxx; mx++) {
			for (int my = 0; my < maxy; my++) {
				ge.set(mx + x, my + y, GraphicsEngine.blend(
						ge.get(mx + x, my + y), get(mx, my)));
			}
		}
	}

	public void set(int x, int y, Color c) {
		if (x < 0 || y < 0 || x >= map.length || y >= map[0].length) {
			return;
		}
		if (map[x][y] == null) {
			map[x][y] = c;
			return;
		}

		map[x][y] = GraphicsEngine.blenda(map[x][y], c);
	}

	public void setraw(int x, int y, Color c) {
		if (x < 0 || y < 0 || x >= map.length || y >= map[0].length) {
			return;
		}

		map[x][y] = c;
	}

	public Color get(int x, int y) {
		if (x < 0 || y < 0 || x >= map.length || y >= map[0].length) {
			return null;
		}
		return map[x][y];
	}
}
