package model;

import java.awt.Color;

import mutil.*;

/**
 * 
 * pickup() return codes: 0/positive: okay. (index of slot) -1[egative]: error.
 */

public class EntityPlayer extends EntitySprite {
	private int hp, maxhp;
	private int exp, maxexp;
	private int level;
	private Inventory inv;
	private Inventory palette;
	private Inventory armor;
	private int palette_select; // which slot is selected in the palette.

	public EntityPlayer(World in, double x, double y) {
		super(in, x, y);
		levelup(1);
		inv     = new Inventory(8*4);
		palette = new Inventory(8*1);
		armor   = new Inventory(1*4);
	}

	public boolean isOpaque() { return true; }
	public double insL() { return 0.2; }
	public double insR() { return 0.2; }
	public double insT() { return 0.2; }
	public double insB() { return 0.2; }
	
	public int getPaletteSelection() { return palette_select; }
	
	@Override
	public int act_Primary(double x, double y) {
		return 0;
	}
	
	public Inventory getInv() {
		return inv;
	}
	public Inventory getPalette() {
		return palette;
	}
	public Inventory getArmor() {
		return armor;
	}

	public int pickup(Object wut) {
		int ret;

		if ((ret = palette.push(wut)) < 0 && (ret = inv.push(wut)) < 0) {
			return -1;
		}

		return ret;
	}

	public void die() {
		System.out.printf("WORLD [Entity %d] (EntityPlayer) died!\n", eid);
		destroyme();
	}
	public void levelup(int levelnew) {
		level = levelnew;

		maxexp = 70 + 20 * (level - 1);
		exp = 1;
		maxhp = 100 + 10 * (level - 1);
		if (level == 1) hp = maxhp; else hp += maxhp / 10;
		
		if (hp > maxhp) {
			hp = maxhp;
		}
	}
	public int hurt(int ouchies) {
		this.hp -= ouchies;
		if (hp < 0) {
			hp = 0;
			die();
		}
		return hp;
	}
	public int getHP() { return hp; }
	public int getMaxHP() { return maxhp; }
	public double getHPRatio() { return (double) maxhp / (double) hp; }
	public int getExp() { return exp; }
	public int getMaxExp() { return maxexp; }
	public double getExpRatio() { return (double) exp / (double) maxexp; }
	public int getLevel() { return level; }
	
	public void move(double x, double y) {
		
		
		
		Block b = world.get((int) x, (int) y);

		//System.out.printf("DEBUG [model.EntityPlayer move(..)] LL=%f TT=%f W=%f H=%f -> x-L=%f y-T=%f\n", insLL(), insTT(), insWidth(), insHeight(), x+insL(), y+insT());
		Entity collidee;

		if ((collidee=world.raytrace(insLL(), insTT(), insWidth(),insHeight(), x+insL(),y+insT())) != null) {
			System.out.println("oh yeah");
			new EntityAnimation(world, collidee.insLL(), collidee.insTT(), Color.red, collidee.insWidth(), collidee.insHeight());
			return;
		}
		super.move(x, y);
	}

	@Override
	public void update(double delta) {
		super.update(delta);
		
		if (speed != 0)
			this.speed = speed/((10*delta)+1);
		if (speed < 0.000001) speed = 0;
	}
	
}















