package trace2d;

public class TraceRect {
	public float ax, ay, bx, by;
	/**
	 * Creates a TraceRect based on supplied coordinates.
	 * @param ax Top-left x.
	 * @param ay Top-left y.
	 * @param bx Bottom-right x.
	 * @param by Bottom-right y.
	 * @return
	 */
	public static TraceRect fromPoints(float ax, float ay, float bx, float by) {
		return new TraceRect(
				Math.min(ax, bx), Math.min(ay, by),
				Math.max(ax, bx), Math.max(ay, by));
	}
	
	/**
	 * Tests whether or not a point is inside the TraceRect.
	 * @param px Point x.
	 * @param py Point y.
	 * @return Is inside ? true : false.
	 */
	public boolean testPoint(float px, float py) {
		return px >= ax && py >= ay && px <= bx && py <= by; 
	}
	
	/**
	 * Tests whether or not this rect intersects with another TraceRect.
	 * @param other Other TraceRect
	 * @return Collides ? true : false.
	 */
	public boolean Intersects(TraceRect other) {
		if ( Math.abs(ax + bx - other.ax - other.bx) < (bx - ax + other.bx - other.ax) )
			if ( Math.abs(ay + by - other.ay - other.by) < (by - ay + other.by - other.ay) )
					return true;
		return false;
	}
	
	public TraceRect() {
		ax = ay = bx = by = 0;
	}
	public TraceRect(float ax, float ay, float bx, float by) {
		this.ax = ax;
		this.ay = ay;
		
		this.bx = bx;
		this.by = by;
	}
}