package view.gui;

import view.GraphicsEngine;

public class WindowManager implements GraphicsEngine.KeyL, GraphicsEngine.MouseL, GraphicsEngine.MouseMotionL {
	// const
	// ---

	// type
	public enum WindowState {
		Normal, Hold
	} // Tray. Maximized. Minimized. etc

	private class Node<T> {
		public T value;
		public Node<T> next, prev;

		public Node(T value) {
			this.value = value;
		}
	}

	interface KeyL {
		boolean eventKeyCode(int code);

		boolean eventKeyChar(char chr);
	}

	interface MouseL {
		boolean eventMouseClick(int x, int y);
	}

	interface MouseMotionL {
		boolean eventMouseMove(int x, int y);

		boolean eventMouseDrag(int x, int y);
	}

	// var
	private Node<Window> begin, end; // linked list
	private GraphicsEngine ge;
	private Window focus, draglock; // draglock = component which didnt accept
									// builtin drag
	private boolean dragging; // is the focus dragging ? (dragging<->focus
								// DIFFERENT THAN draglock)

	// priv
	private Node<Window> getNodeByWindow(Window which) {
		Node<Window> cur = begin;

		while ((cur = cur.next) != null) {
			if (cur.value == which) {
				return cur;
			}
		}
		return null;
	}

	private Window getWindowAt(int x, int y) {
		Node<Window> cur = end; // REVERSED TRAVERSAL. not list (end = topmost)

		do {
			Window w = cur.value;

			if (w == null) {
				continue;
			}

			if (w.state == WindowState.Normal && x >= w.x && y >= w.y
					&& x < w.x + w.xs && y < w.y + w.ys) {
				return w;
			} // yay!!!!!
			if (w.state == WindowState.Hold && x >= w.x && y >= w.y
					&& x < w.x + w.xs && y < w.y + Window.frametop) {
				return w;
			} // yay!!!!!
				// more states here...

		} while ((cur = cur.prev).prev != null); // cur.prev being "begin". then
													// (begin).prev == null.

		return null; // none of them matched
	}

	private void pokeAll() {
		Node<Window> cur = begin;

		while ((cur = cur.next) != null) {
			cur.value.poke();
		}
	}

	// init
	public void initGE(GraphicsEngine ge) {
		this.ge = ge;
	}

	public WindowManager() {
		dragging = false;
		begin = new Node<Window>(null);
		end = begin;

		focus = null;
		ge = null;
	}

	// set
	// ---

	// get
	public GraphicsEngine getGE() {
		return this.ge;
	}

	public Window getFocus() {
		return focus;
	} // return focused window

	public boolean isdragging() {
		return dragging;
	}

	// core
	public void render() {
		Node<Window> cur = begin;

		while ((cur = cur.next) != null) {
			cur.value.render(ge);
		}
	}

	public void rebuild() {
		System.out.println("[view.WindowManager rebuild];"); // TODO
	}

	public void requestredraw() {
		System.out.println("[view.gui.WindowManager requestredraw]!"); // TODO
	}

	// ===
	public void add(Window wut) {

		/*
		 * from 110624: reversed linkedlist Node<Window> node = new
		 * Node<Window>(wut); node.prev = begin; node.next = begin.next;
		 * begin.next = node; if (node.next != null) node.next.prev = node;
		 */

		Node<Window> node = new Node<Window>(wut);

		node.next = null;
		node.prev = end;
		end.next = node;
		end = node;
		if (!dragging) {
			bringTop(wut);
		}

	}

	public void remove(Window wut) {
		Node<Window> node = getNodeByWindow(wut);

		if (node == null) {
			return;
		}

		if (node.prev != null) {
			node.prev.next = node.next;
		}
		if (node.next != null) {
			node.next.prev = node.prev;
		}
		node.next = null;
		node.prev = null;
		node.value = null;
	}

	public boolean exists(Window which) {
		Node<Window> cur = begin;

		while ((cur = cur.next) != null) {
			if (cur.value == which) {
				return true;
			}
		}
		return false;
	}

	public boolean holdAdd(Window wut) {

		return false;
	}

	public boolean holdPop(Window wut) {

		return false;
	}

	// / "try to gain focus"
	public void bringTop(Window which) {
		Node<Window> result = getNodeByWindow(which);

		if (result == null) {
			return;
		}
		if (result == end) {
			focus = which;
			return;
		}

		/*
		 * from 110624: reversed linkedlist if (result.prev != null)
		 * result.prev.next = result.next; if (result.next != null)
		 * result.next.prev = result.prev;
		 * 
		 * result.next = begin.next; result.prev = begin; begin.next.prev =
		 * result; begin.next = result;
		 * 
		 * focus = which;
		 */

		/*
		 * if (result.prev != null) { result.prev.next = result.next; } // cut
		 * if (result.next != null) { result.next.prev = result.prev; } // cut
		 * 
		 * begin.next.prev = result; result.prev = begin; result.next =
		 * begin.next; begin.next = result;
		 * 
		 * focus = which;
		 */

		end.next = result;

		if (result.prev != null) {
			result.prev.next = result.next;
		}
		if (result.next != null) {
			result.next.prev = result.prev;
		}
		result.next = null;
		result.prev = end;
		end = result;
		focus = end.value;

	}

	// event
	public boolean mouseMoved(int x, int y) {
		if (dragging) {
			return true;
		}
		if (draglock != null) {
			return true;
		}

		Window found = getWindowAt(x, y);

		if (found == null) {
			return false;
		}
		found.eventMouseMove(x - found.x, y - found.y);
		return true;

		/*
		 * 110625 Node<Window> cur = begin; while ((cur=cur.next) != null) {
		 * Window w = cur.value; if (x<w.x || y<w.y || x>=w.x+w.xs ||
		 * y>=w.y+w.ys) continue; w.eventMouseMove(x-w.x, y-w.y); return true; }
		 * return false;
		 */
	}

	public boolean mouseDragged(int x, int y) {
		if (dragging) {
			if (focus == null) {
				dragging = false;
				return false;
			}
			dragging = focus.drag(x - focus.x, y - focus.y); // implies dropping
																// TODO
			return true;
		}
		if (draglock != null) {
			if (x >= draglock.x + Window.frameleft
					&& y >= draglock.y + Window.frametop
					&& x < draglock.x + draglock.xs - Window.frameright
					&& y < draglock.y + draglock.ys - Window.framebottom) {
				draglock.eventMouseDrag(x - draglock.x, y - draglock.y);
			}

			return true;
		}
		return false;

		// should be handled specially via somehting like
		// mouseDown and MouseUp, as you might not want to drag something
		// over every window any give the dragged item to the disposal

		/*
		 * Node<Window> cur = begin; while ((cur=cur.next) != null) { Window w =
		 * cur.value; if (x<w.x || y<w.y || x>=w.x+w.xs || y>=w.y+w.ys)
		 * continue; bringTop(w); w.eventMouseDrag(x-w.x, y-w.y); return true; }
		 * return false;
		 */
	}

	public boolean mouseClicked(int x, int y) {
		Window found = getWindowAt(x, y);

		if (found == null) {
			focus = null;
			return true;
		}
		found.eventMouseClick(x - found.x, y - found.y);
		return true;

		/*
		 * 110625 Node<Window> cur = begin; while ((cur=cur.next) != null) {
		 * Window w = cur.value; if (w==null) continue;
		 * 
		 * if (w.state==WindowState.Hold && x>=w.x && y>=w.y && x<w.x+w.xs &&
		 * y<w.y+Window.frametop) { // TODO make coords be in hold area always
		 * bringTop(w); return w.eventMouseClick(x-w.x, y-w.y); }
		 * 
		 * if (w.state==WindowState.Normal && x>=w.x && y>=w.y && x<w.x+w.xs &&
		 * y<w.y+w.ys) { bringTop(w); return w.eventMouseClick(x-w.x, y-w.y); }
		 * 
		 * // ** if (x>=w.x && y>=w.y && x<w.x+w.xs && y<w.y+w.ys);
		 * 
		 * if (w.state == WindowState.Hold) if (x<w.x || y<w.y || x>=w.x+w.xs ||
		 * y>=w.y+Window.frametop) continue; // case Hold else if (x<w.x ||
		 * y<w.y || x>=w.x+w.xs || y>=w.y+w.ys ) continue; // default Normal
		 * w.eventMouseClick(x-w.x, y-w.y); bringTop(w); return true;* // }
		 * focus = null; return false;
		 */
	}

	public boolean keyTyped(char letter) {
		if (focus != null) {
			return focus.eventKeyChar(letter);
			// return true;
		} else {
			return false;
		}
	}

	public boolean keyPressed(int code) {
		if (focus != null) {
			return focus.eventKeyCode(code);
			// return true;
		} else {
			return false;
		}
	}

	public boolean mouseDown(int x, int y) {
		if (dragging) {
			return true;
		}
		if (draglock != null) {
			return true;
		}

		Window found = getWindowAt(x, y);

		if (found == null) {
			focus = null;
			return false;
		}
		if (found.startdrag(x - found.x, y - found.y)) {
			dragging = true;
		} else {
			draglock = found;
		}
		bringTop(found);
		return true;

		/*
		 * Node<Window> cur = begin; while ((cur=cur.next) != null) { Window w =
		 * cur.value; if (x<w.x || y<w.y || x>=w.x+w.xs || y>=w.y+w.ys)
		 * continue; if (w.startdrag(x-w.x, y-w.y)) { dragging = true;
		 * bringTop(w); } else { draglock = w; } return true; } return false;
		 */
	}

	public boolean mouseUp(int x, int y) {
		draglock = null;
		if (!dragging) {
			return false;
		}
		if (focus == null) {
			dragging = false;
			return false;
		} // window got killed or defocused. reset drag

		focus.stopdrag(x - focus.x, y - focus.y);
		dragging = false;

		return false;
	}

	@Override
	public boolean keyReleased(int code) {
		// TODO Auto-generated method stub
		return false;
	}

}
